"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable max-len */
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = require("child_process");
const vitest_1 = require("vitest");
const create_1 = __importDefault(require("../create"));
const utils_js_1 = require("../utils.js");
const testUtilsDir = path_1.default.join(__dirname, 'test-utils', 'scaffolds');
vitest_1.vi.mock('child_process');
// Shoutout to chatgpt for this function
const makeTree = (directoryPath, indent = 0) => {
    const files = fs_1.default
        .readdirSync(directoryPath)
        // sorting to make directory read sequence same
        .sort();
    let tree = '';
    files.forEach((file) => {
        const filePath = path_1.default.join(directoryPath, file);
        const stats = fs_1.default.statSync(filePath);
        const isDirectory = stats.isDirectory();
        tree += ' '.repeat(indent * 2);
        tree += isDirectory ? '├── ' + file + '\n' : '└── ' + file + '\n';
        if (isDirectory) {
            tree += makeTree(filePath, indent + 1);
        }
    });
    return tree;
};
(0, vitest_1.beforeAll)(() => {
    (0, utils_js_1.deleteDir)(testUtilsDir);
});
(0, vitest_1.afterAll)(() => {
    (0, utils_js_1.deleteDir)(testUtilsDir);
});
(0, vitest_1.describe)('create', () => {
    (0, vitest_1.beforeEach)(() => {
        vitest_1.vi.spyOn(process, 'cwd').mockReturnValue(testUtilsDir);
        // @ts-expect-error: only mocking what is used
        child_process_1.spawn.mockImplementation(() => {
            const mockChildProcess = {
                stdout: { on: vitest_1.vi.fn() },
                stderr: { on: vitest_1.vi.fn() },
                on: vitest_1.vi.fn()
            };
            return mockChildProcess;
        });
    });
    (0, vitest_1.afterEach)(() => {
        vitest_1.vi.resetAllMocks();
        (0, utils_js_1.deleteDir)(path_1.default.join(testUtilsDir, 'hello-abell'));
    });
    (0, vitest_1.test)('should create project with default template', () => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, create_1.default)('hello-abell', {
            installer: 'skip',
            template: 'default'
        });
        (0, vitest_1.expect)(child_process_1.spawn).not.toBeCalled();
        (0, vitest_1.expect)(fs_1.default.existsSync(path_1.default.join(testUtilsDir, 'hello-abell'))).toBe(true);
        (0, vitest_1.expect)(makeTree(path_1.default.join(testUtilsDir, 'hello-abell')))
            .toMatchInlineSnapshot(`
        "├── _components
          └── global.meta.abell
          └── navbar.abell
        └── about.abell
        ├── client-assets
          └── global.css
          └── index-client.js
        └── config.js
        └── index.abell
        └── package.json
        ├── public
          └── abell-logo.svg
        "
      `);
    }));
    (0, vitest_1.test)('should create project with default template and bun installer', () => __awaiter(void 0, void 0, void 0, function* () {
        yield (0, create_1.default)('hello-abell', {
            installer: 'bun',
            template: 'default'
        });
        const expectedCommand = 'bun install';
        const commandArgs = (0, utils_js_1.windowsifyCommand)(expectedCommand).split(' ');
        (0, vitest_1.expect)(child_process_1.spawn).toBeCalledWith(commandArgs[0], [commandArgs[1]], {
            cwd: path_1.default.join(testUtilsDir, 'hello-abell'),
            stdio: 'inherit'
        });
        const packageJSONPath = path_1.default.join(testUtilsDir, 'hello-abell', 'package.json');
        const packageJSON = JSON.parse(fs_1.default.readFileSync(packageJSONPath, 'utf-8'));
        (0, vitest_1.expect)(packageJSON.name).toBe('hello-abell');
        (0, vitest_1.expect)(packageJSON.scripts).toMatchInlineSnapshot(`
      {
        "dev": "bunx --bun abell dev",
        "generate": "bunx --bun abell generate",
      }
    `);
    }));
});
